<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://welaunch.io/plugins/woocommerce-ultimate-pricing/
 * @since      1.0.0
 *
 * @package    WooCommerce_delivery
 * @subpackage WooCommerce_delivery/public
 */
class WooCommerce_Ultimate_Pricing_Discount_Rules extends WooCommerce_Ultimate_Pricing {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	protected $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of this plugin.
	 */
	protected $version;

	/**
	 * options of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      array    $options
	 */
	protected $options;


	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) 
	{
		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->data = array();
	}

	/**
	 * Enqueu Styles
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https://welaunch.io
	 * @return  [type]                       [description]
	 */
	public function enqueue_styles() 
	{
		global $woocommerce_ultimate_pricing_options;
		$this->options = $woocommerce_ultimate_pricing_options;

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( dirname( __FILE__ ) ) . 'assets/css/woocommerce-ultimate-pricing-public.css', array(), $this->version, 'all' );
	}

	/**
	 * Enque JS SCripts
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https://welaunch.io
	 * @return  [type]                       [description]
	 */
	public function enqueue_scripts() 
	{
		return false;
	}
	
	public function init()
	{

		global $woocommerce_ultimate_pricing_options;
		$this->options = $woocommerce_ultimate_pricing_options;

		if (!$this->get_option('enable')) {
			return false;
		}

		$this->user_id = get_current_user_id();
		$this->discountsApplied = array();

		if(is_admin()) { 
			return false;
		}


		// Discount Filters
		add_filter( 'woocommerce_product_get_price', array($this, 'apply_discounts'), 30, 2);
		add_filter( 'woocommerce_product_get_sale_price', array($this, 'apply_discounts'), 30, 2);
		add_filter( 'woocommerce_product_variation_get_sale_price', array($this, 'apply_discounts'), 30, 2);
		add_filter( 'woocommerce_product_variation_get_price', array($this, 'apply_discounts'), 30, 2);
		add_filter( 'woocommerce_get_price_html', array($this, 'show_discount_tooltip'), 30, 2);

		// Variation price gets saved in transients ???
		add_filter( 'woocommerce_variation_prices_price', array($this, 'apply_discounts'), 30, 2);
		add_filter( 'woocommerce_variation_prices_sale_price', array($this, 'apply_discounts'), 30, 2);
	}

	public function apply_discounts($price, $product)
	{

		if(!$product) {
			return $price;
		}

	    if(empty($price)) {
	    	return $price;
	    }

	    $productId = $product->get_id();

		if(!isset($this->discountsApplied[$productId])) {
			$this->discountsApplied[$productId] = array(
				'rules' => array(),
				'fixedDiscountTotal' => 0,
				'percentageDiscountTotal' => 0,
				'discountRateTotal' => 0,
			);	
		}

		// Already applied rules - do not apply again performance
		// if(isset($this->discountsApplied[$productId]['applied']) && $this->discountsApplied[$productId]['applied']) {
		// 	return $price;
		// }

		$percentageDiscountTotal = 0;
		$fixedDiscountTotal = 0;
		
		$role = $this->get_user_role($this->user_id);
		
		// Custom User Role Price
		if($this->get_option('customUserRolePrices')) {
			$customUserRolePrices = $product->get_meta('woocommerce_ultimate_pricing_prices');
			if($customUserRolePrices && isset($customUserRolePrices[$role]) && !empty($customUserRolePrices[$role])) {

				$originalPrice = $price;
				$newPrice = $customUserRolePrices[$role];
				$discountRateTotal = $originalPrice - $newPrice;

				$this->discountsApplied[$productId]['applied'] = true;
				$this->discountsApplied[$productId]['fixedDiscountTotal'] = $discountRateTotal;
				$this->discountsApplied[$productId]['percentageDiscountTotal'] = $percentageDiscountTotal;
				
				// Will make sure old sale prices will be used for regular prices
				$product->set_regular_price($originalPrice);
				$this->discountsApplied[$productId]['discountRateTotal'] = $originalPrice - $newPrice;

				return $newPrice;
			}
		}

		$discountRulesCount = $this->get_option('discountRulesCount');
		for ($i=1; $i <= $discountRulesCount; $i++) { 

			if(!$this->get_option('discountRule' . $i . 'Enabled')) {
				continue;
			}

		    $roles = $this->get_option('discountRule' . $i . 'Roles');
		    if(!empty($roles)) {
			   		
			   	// No Role or Guest
			   	if(!$role) {
			   		continue;
			   	}

				if(!in_array($role, $roles)) {
					continue;
				}
		    }

		    $dateRange = $this->get_option('discountRule' . $i . 'DateRange');
		    if(!empty($dateRange)) {
		    	$dateRange = explode(' to ', $dateRange);
		    	$startDate = $dateRange[0];
		    	$endDate = $dateRange[1];
				if(!empty($startDate) && !empty($endDate)) {
					$currentDateObject = new DateTime();
					$startDateObject = new DateTime($startDate);
					$endDateObject = new DateTime($endDate . ' 23:59');
				    if($currentDateObject < $startDateObject || $currentDateObject > $endDateObject) {
				        continue;
				    }
				}
		    }

		    $categories = $this->get_option('discountRule' . $i . 'Categories');
		    if(!empty($categories)) {
			    if( !has_term($categories, 'product_cat', $productId) ) {
			        continue;
			    }
		    }

		    $products = $this->get_option('discountRule' . $i . 'Products');
		    if(!empty($products)) {
			    if( !in_array($productId, $products)) {
			        continue;
			    }
		    }

		    $tags = $this->get_option('discountRule' . $i . 'Tags');
		    if(!empty($tags)) {
			    if( !has_term($tags, 'product_tag', $productId) ) {
			        continue;
			    }
		    }

		    $reviews = $this->get_option('discountRule' . $i . 'Reviews');
		    if(!empty($reviews)) {
			    $userReviews = $this->get_user_review_count($this->user_id);
			    if($userReviews < $reviews) {
			    	continue;
			    }
		    }

		    $orders = $this->get_option('discountRule' . $i . 'Orders');
		    if(!empty($orders)) {
			    $userOrders = $this->get_user_order_count($this->user_id);
			    if($userOrders < $orders) {
			    	continue;
			    }
		    }

		    $this->discountsApplied[$productId]['applied'] = true;

		    if(!$this->get_option('discountRule' . $i . 'Addable')) {

		    	$fixedDiscount = 0;
				$percentageDiscountTotal = 0;
				$this->discountsApplied[$productId]['rules'] = array();

				if($this->get_option('discountRule' . $i . 'Type') == "percentage") {
					$percentageDiscount = $this->get_option('discountRule' . $i . 'PercentageDiscount');
					$percentageDiscountTotal = $percentageDiscount;
				} else {
					$fixedDiscount = $this->get_option('discountRule' . $i . 'FixedDiscount');
					$fixedDiscountTotal = $fixedDiscount;
				}

	    	} else {

				if($this->get_option('discountRule' . $i . 'Type') == "percentage") {
					$percentageDiscount = $this->get_option('discountRule' . $i . 'PercentageDiscount');
					$percentageDiscountTotal = $percentageDiscountTotal + $percentageDiscount;
				} else {
					$fixedDiscount = $this->get_option('discountRule' . $i . 'FixedDiscount');
					$fixedDiscountTotal = $fixedDiscountTotal + $fixedDiscount;
				}
	    	}

			if(!in_array($i, $this->discountsApplied[$productId]['rules'])) {
				$this->discountsApplied[$productId]['rules'][] = $i;
			}
		}

		if($fixedDiscountTotal <= 0 && $percentageDiscountTotal <= 0) {
			return $price;
		}

		$this->discountsApplied[$productId]['fixedDiscountTotal'] = $fixedDiscountTotal;
		$this->discountsApplied[$productId]['percentageDiscountTotal'] = $percentageDiscountTotal;

		$originalPrice = $price;
		// Will make sure old sale prices will be used for regular prices
		$product->set_regular_price($originalPrice);

		if($percentageDiscountTotal > 0) {
			$newPrice = $price * ( 1 - intval($percentageDiscountTotal) / 100);
			if($newPrice <= 0) {
				$price = 0;
			} else {
				$price = $newPrice;
			}
		}

		if($fixedDiscountTotal > 0) {
			$newPrice = $price - floatval($fixedDiscountTotal);
			if($newPrice <= 0) {
				$price = 0;
			} else {
				$price = $newPrice;
			}
		}

		$this->discountsApplied[$productId]['discountRateTotal'] = $originalPrice - $newPrice;

	   	return $price;
	}

	public function show_discount_tooltip($price_html, $product)
	{
		if(!$this->get_option('tooltipEnabled')) {
			return $price_html;
		}

		if(!$product) {
			return $price_html;	
		}

		$productId = $product->get_id();

		if(!isset($this->discountsApplied[$productId]) || empty($this->discountsApplied[$productId])) {
			return $price_html;
		}

	    // Woo saves variation prices in a transient
	    // have to modify and rebuild original price_html code
	    // see: class-wc-product-variable.php > get_price_html
	    if($product->is_type('variable')) {

			$prices = $product->get_variation_prices( true );

			if ( !empty( $prices['price'] ) ) {

				$min_price     = current( $prices['price'] ) * (1 - intval($this->discountsApplied[$productId]['discountRateTotal']) / 100);
				$max_price     = end( $prices['price'] ) * (1 - intval($this->discountsApplied[$productId]['discountRateTotal']) / 100);
				$min_reg_price = current( $prices['regular_price'] );
				$max_reg_price = end( $prices['regular_price'] );

				if ( $min_price !== $max_price ) {
					$price = wc_format_price_range( $min_price, $max_price );
				} elseif ( $product->is_on_sale() && $min_reg_price === $max_reg_price ) {
					$price = wc_format_sale_price( wc_price( $max_reg_price ), wc_price( $min_price ) );
				} else {
					$price = wc_price( $min_price );
				}

				$price_html = apply_filters( 'woocommerce_variable_price_html', $price . $product->get_price_suffix(), $product );

				// On single products not show the discount applied stuff
				if(is_product()) {
					return $price_html;
				}
			}
	    } 

		$discountRulesTexts = array();
		$discountRuleTextTemplate = $this->get_option('tooltipSingleDiscountRuleText');

		foreach ($this->discountsApplied[$productId]['rules'] as $discountRuleId ) {

			$discountTooltipText = $this->get_option('discountRule' . $discountRuleId . 'TooltipText');
			if(!empty($discountTooltipText)) {
				$discountRulesTexts[] = $discountTooltipText;
			} else {
				$discountTitle = $this->get_option('discountRule' . $discountRuleId . 'Title');
				if( $this->get_option('discountRule' . $discountRuleId . 'Type') == "percentage") {
					$discountRate = intval( $this->get_option('discountRule' . $discountRuleId . 'PercentageDiscount') ) . '%';
				} else {
					$discountRate = wc_price( floatval( $this->get_option('discountRule' . $discountRuleId . 'FixedDiscount') ) );
				}

				$discountRulesTexts[] = str_replace(array('{{discountRate}}', '{{discountTitle}}'), array($discountRate, $discountTitle), $discountRuleTextTemplate);
			}
		}

		$discountRulesText = "";
		if(count( $discountRulesTexts) > 1) {
			$discountRulesText = implode($this->get_option('tooltipDiscountRulesAppendix'), $discountRulesTexts);
		} elseif(!empty($discountRulesTexts)) {
			$discountRulesText = $discountRulesTexts[0];
		}
		

	    $text = $this->get_option('tooltipText');
	    if(empty($text)) {
	    	return $price_html;
	    }

	    $text = str_replace(
	    	array(
				'{{fixedDiscountTotal}}',
				'{{percentageDiscountTotal}}',
				'{{discountRateTotal}}',
				'{{discountRulesText}}',
			),
	    	array(
				wc_price( $this->discountsApplied[$productId]['fixedDiscountTotal'] ),
				$this->discountsApplied[$productId]['percentageDiscountTotal'] . '%',
				wc_price( $this->discountsApplied[$productId]['discountRateTotal'] ),
				$discountRulesText,
	    	),
	    	$text
	    );

	    if(!$product->is_on_sale()) {
   			$price_html = '<del>' . wc_price($product->get_regular_price()) . '</del>' . $price_html;
   		}

	   	$price_html .= '<span class="woocommerce-ultimate-pricing-tooltip" woocommerce-ultimate-pricing-tooltip="' . strip_tags( $text ) . '">' . $this->get_option('tooltipIcon') . '</span>';

	   	return $price_html;
	}

}