<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://welaunch.io/plugins/woocommerce-ultimate-pricing/
 * @since      1.0.0
 *
 * @package    WooCommerce_delivery
 * @subpackage WooCommerce_delivery/public
 */
class WooCommerce_Ultimate_Pricing_Public extends WooCommerce_Ultimate_Pricing {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	protected $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of this plugin.
	 */
	protected $version;

	/**
	 * options of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      array    $options
	 */
	protected $options;


	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) 
	{
		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->data = array();
	}

	/**
	 * Enqueu Styles
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https://welaunch.io
	 * @return  [type]                       [description]
	 */
	public function enqueue_scripts_styles() 
	{
		global $woocommerce_ultimate_pricing_options;
		$this->options = $woocommerce_ultimate_pricing_options;

		if (!$this->get_option('enable')) {
			return false;
		}

		wp_enqueue_style( $this->plugin_name . '-public', plugin_dir_url( dirname( __FILE__ ) ) . 'assets/css/woocommerce-ultimate-pricing-public.css', array(), $this->version, 'all' );

        if($this->get_option('productDiscountsBadgeEnabled')) {

			$productDiscountsBadgeBackgroundColor = $this->get_option('productDiscountsBadgeBackgroundColor');
			$productDiscountsBadgeTextColor = $this->get_option('productDiscountsBadgeTextColor');

			echo '<style>
			.woocommerce-ultimate-pricing-badge, .woocommerce-ultimate-pricing-badge span.amount {
				 background-color: ' . $productDiscountsBadgeBackgroundColor . ' !important;
				 color: ' . $productDiscountsBadgeTextColor . ' !important;
			 }
			 
			 .woocommerce-ultimate-pricing-badge:after {
			 	border-right-color: ' . $productDiscountsBadgeBackgroundColor . ' !important;
			 }
			 </style>
			 ';
		 }

		// Just tiered pricing JS
		if($this->get_option('performanceOnlyWooPages') && !is_product()) {
			return;
		}
		
		wp_enqueue_script( $this->plugin_name . '-public', plugin_dir_url( dirname( __FILE__ ) ). 'assets/js/woocommerce-ultimate-pricing-public.js', array('jquery'), $this->version, true);

		$decimalNumbers = (int) get_option('woocommerce_price_num_decimals');
		$decimalNumbersString = '';

		if($decimalNumbers > 0) {
			$decimalNumbersString = str_pad(0, $decimalNumbers, '0');
		}

		$forJS = array(
			'ajax_url' => admin_url('admin-ajax.php'),
			'price_template' => wc_price(0),
			'tiered_pricing' => $this->get_option('tieredPricingEnabled'),
			'wc_decimal_seperator' => wc_get_price_decimal_separator(),
			'wc_decimal_num' => $decimalNumbersString,
		);

        wp_localize_script($this->plugin_name . '-public', 'woocommerce_ultimate_pricing_options', $forJS);

	}

	
	public function init()
	{
		global $woocommerce_ultimate_pricing_options;
		$this->options = $woocommerce_ultimate_pricing_options;


	}

}